/*
 * wizInterface.c
 *
 */

#include "wizInterface.h"
#include "httpUtil.h"

extern SPI_HandleTypeDef hspi1;
#define WIZ_SPI_HANDLE	&hspi1

#ifdef USE_DHCP
static uint8_t dhcp_buffer[1024];
static uint16_t dhcp_retry = 0;
#endif

//network information
//wiz_NetInfo netInfo = {
//		.mac = { 0x70, 0xff, 0x76, 0x1d, 0xc6, 0xfc },	// Mac address
//		.ip = { 192, 168, 1, 180 },						// IP address
//		.sn = { 255, 255, 255, 0 },						// Subnet mask
//		.gw = { 192, 168, 1, 1 },						// Gateway address
//		.dns 	= {8, 8, 8, 8},							// DNS server
//		.dhcp 	= NETINFO_STATIC						// DHCP enable / disable
//		};

wiz_NetInfo netInfo = {
							.mac 	= {0x00, 0x08, 0xdc, 0xab, 0xcd, 0xef},	// Mac address
                            .ip 	= {192, 168, 0, 115},					// IP address
                            .sn 	= {255, 255, 255, 0},					// Subnet mask
                            .gw 	= {192, 168, 0, 1},						// Gateway address
                            .dns 	= {8, 8, 8, 8},							// DNS server
                            .dhcp 	= NETINFO_STATIC };						// DHCP enable / disable

wiz_NetTimeout timeout = {
		.retry_cnt = 3, 		//RCR = 3
		.time_100us = 1000};    //500ms

void WIZ_SPI_Select(void)
{
	HAL_GPIO_WritePin(WIZ_SPI1_CS_GPIO_Port, WIZ_SPI1_CS_Pin, GPIO_PIN_RESET);
}

void WIZ_SPI_Deselect(void)
{
	HAL_GPIO_WritePin(WIZ_SPI1_CS_GPIO_Port, WIZ_SPI1_CS_Pin, GPIO_PIN_SET);
}

void WIZ_SPI_TxByte(uint8_t byte)
{
	HAL_SPI_Transmit(WIZ_SPI_HANDLE, &byte, 1, HAL_MAX_DELAY);
}

uint8_t WIZ_SPI_RxByte(void)
{
	uint8_t ret;
	HAL_SPI_Receive(WIZ_SPI_HANDLE, &ret, 1, HAL_MAX_DELAY);
	return ret;
}

void WIZ_SPI_TxBuffer(uint8_t *buffer, uint16_t len)
{
	HAL_SPI_Transmit(WIZ_SPI_HANDLE, buffer, len, HAL_MAX_DELAY);
}

void WIZ_SPI_RxBuffer(uint8_t *buffer, uint16_t len)
{
	HAL_SPI_Receive(WIZ_SPI_HANDLE, buffer, len, HAL_MAX_DELAY);
}


bool WIZ_ChipInit(void) {

	/*!< set socket size to 16KB, only one socket is being used */
	uint8_t memsize[2][8] = { { 16, 16, 0, 0, 0, 0, 0, 0 }, { 0, 0, 0, 0, 0, 0, 0, 0 } };

	/*!< De-select the FLASH: Chip Select high */
	WIZ_SPI_Deselect();

	/*!< reset Ethernet shield */
	HAL_GPIO_WritePin(WIZ_RESET_GPIO_Port, WIZ_RESET_Pin, GPIO_PIN_RESET);
	HAL_Delay(1000);
	HAL_GPIO_WritePin(WIZ_RESET_GPIO_Port, WIZ_RESET_Pin, GPIO_PIN_SET);
	HAL_Delay(1000);

	// Wiz-net : register SPI functions
	reg_wizchip_cs_cbfunc(WIZ_SPI_Select, WIZ_SPI_Deselect);
	reg_wizchip_spi_cbfunc(WIZ_SPI_RxByte, WIZ_SPI_TxByte);
	// Wiz-net : if using DMA register SPI functions
	reg_wizchip_spiburst_cbfunc(WIZ_SPI_RxBuffer, WIZ_SPI_TxBuffer);

	/* wiz-chip : initialize */
	if(ctlwizchip(CW_INIT_WIZCHIP, (void*) memsize)){
		myprintf("WIZCHIP Initialized fail.\r\n");
		return false;
	}

	/* wiz-chip : check version register */
	uint8_t version = getVERSIONR();
	if(version != 0x04)
	{
		myprintf("getVERSIONR returns wrong version!\n");
		return false;
	}

	/* wiz-chip : check PHY status */
	wiz_PhyConf phyConf;
	phyConf.by   =0;
	phyConf.mode =0;
	wizphy_getphystat(&phyConf);
	myprintf("PHY Status > conf.raw->={0x%2X} , conf.by = {%d}, conf.mode={%d}, conf.speed={%d}, conf.duplex={%d}\n",
			phyConf.raw,phyConf.by, phyConf.mode, phyConf.speed, phyConf.duplex);

	uint8_t tmpstr[6] = {0,};
	ctlwizchip(CW_GET_ID,(void*)tmpstr);
	myprintf("\r\n=======================================\r\n");
	myprintf(" WIZnet %s EVB Demos v%d.%.2d\r\n", tmpstr, version, 00);
	myprintf("=======================================\r\n");

	return true;
}

void WIZ_GetNetworkInfo(void) {
	wiz_NetInfo tmpInfo;

	wizchip_getnetinfo(&tmpInfo);
	myprintf("IP: %03d.%03d.%03d.%03d\nGW: %03d.%03d.%03d.%03d\nNet: %03d.%03d.%03d.%03d\n",
			tmpInfo.ip[0], tmpInfo.ip[1],tmpInfo.ip[2], tmpInfo.ip[3],
			tmpInfo.gw[0], tmpInfo.gw[1], tmpInfo.gw[2], tmpInfo.gw[3],
			tmpInfo.sn[0], tmpInfo.sn[1], tmpInfo.sn[2], tmpInfo.sn[3]);
}

bool WIZ_NetworkInit(void) {
	wiz_NetInfo tmpInfo;
	wiz_NetTimeout tmpTimeout;

#ifdef USE_DHCP
	setSHAR(netInfo.mac); //set MAC address
	DHCP_init(DHCP_SOCKET, dhcp_buffer); //init DHCP
	reg_dhcp_cbfunc(cbIPAddrAssigned, cbIPAddrAssigned, cbIPAddrConfict); //register DHCP callbacks

	//get ip from dhcp server
	dhcp_retry = 0;
	while (!ip_assigned && dhcp_retry < 100000) {
		dhcp_retry++;
		DHCP_run();
	}

	//if dhcp assigned an ip address.
	if (ip_assigned) {
		getIPfromDHCP(netInfo.ip);
		getGWfromDHCP(netInfo.gw);
		getSNfromDHCP(netInfo.sn);
	}
#endif

	//set network information
	wizchip_setnetinfo(&netInfo);

	//get network information
	wizchip_getnetinfo(&tmpInfo);
	myprintf("IP: %03d.%03d.%03d.%03d\nGW: %03d.%03d.%03d.%03d\nNet: %03d.%03d.%03d.%03d\n",
			tmpInfo.ip[0], tmpInfo.ip[1],tmpInfo.ip[2], tmpInfo.ip[3],
			tmpInfo.gw[0], tmpInfo.gw[1], tmpInfo.gw[2], tmpInfo.gw[3],
			tmpInfo.sn[0], tmpInfo.sn[1], tmpInfo.sn[2], tmpInfo.sn[3]);

	if(tmpInfo.mac[0] != netInfo.mac[0] ||
			tmpInfo.mac[1] != netInfo.mac[1] ||
			tmpInfo.mac[2] != netInfo.mac[2] ||
			tmpInfo.mac[3] != netInfo.mac[3])
	{
		myprintf("wizchip_getnetinfo failed.\n");
		return false;
	}

	//set timeout
	ctlnetwork(CN_SET_TIMEOUT,(void*)&timeout);
	ctlnetwork(CN_GET_TIMEOUT, (void*)&tmpTimeout);

	if(tmpTimeout.retry_cnt != timeout.retry_cnt || tmpTimeout.time_100us != timeout.time_100us)
	{
		myprintf("ctlnetwork(CN_SET_TIMEOUT) failed.\n");
		return false;
	}

	return true;
}
